/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.thread;

import java.util.Properties;

import org.junit.Before;
import org.junit.Test;

import com.ebmwebsourcing.easycommons.properties.PropertiesHelper;

import static org.junit.Assert.*;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class ExecutionContextTest {

    @Before
    public void before() {
        ExecutionContext.getProperties().clear();
    }

    @Test
    public void testIsEmptyOnInit() throws Exception {
        assertTrue(ExecutionContext.getProperties().isEmpty()); 
    }
    
    @Test
    public void testIsEmptyAfterPut() throws Exception {
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        assertFalse(ExecutionContext.getProperties().isEmpty());
    }
    
    @Test
    public void testGetWithUnknownKey() throws Exception {
        assertNull(ExecutionContext.getProperties().getProperty("testKey1")); 
    }

    @Test
    public void testPutAndGet() throws Exception {
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        assertEquals("testValue1", ExecutionContext.getProperties().getProperty("testKey1")); 
    }
    
    @Test
    public void testPutTwiceAndGet() throws Exception {
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        assertEquals("testValue1", ExecutionContext.getProperties().getProperty("testKey1")); 
    }
    

    @Test
    public void testUpdateAndGet() throws Exception {
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        assertEquals("testValue1", ExecutionContext.getProperties().getProperty("testKey1")); 
        ExecutionContext.getProperties().setProperty("testKey1", "testValue2");
        assertEquals("testValue2", ExecutionContext.getProperties().getProperty("testKey1")); 
    }
    
    @Test
    public void testClear() throws Exception {
        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
        ExecutionContext.getProperties().setProperty("testKey1", "testValue2");
        ExecutionContext.getProperties().clear();
        assertTrue(ExecutionContext.getProperties().isEmpty()); 
    }
    
    @Test
    public void testPutInParentThreadClearInChildThreadAndGetInChildThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                ExecutionContext.getProperties().setProperty("testKey1", "testParentValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        ExecutionContext.getProperties().setProperty("testKey1", "testChildValue1");
                        assertEquals("testChildValue1", ExecutionContext.getProperties().getProperty("testKey1"));
                        ExecutionContext.getProperties().clear();
                        assertEquals("testParentValue1", ExecutionContext.getProperties().getProperty("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }
    
    @Test
    public void testPutAndGetInSiblingThreads() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
            }
        });
        tt1.start();
        tt1.joinExplosively();
        
        TestThread tt2 = new TestThread(new Runnable() {
            @Override
            public void run() {
                assertNull(ExecutionContext.getProperties().getProperty("testKey1"));
            }
        });
        tt2.start();
        tt2.joinExplosively();
        
    }


    @Test
    public void testPutInParentThreadAndGetInChildThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        assertEquals("testValue1", ExecutionContext.getProperties().getProperty("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

    
    

    @Test
    public void testPutInChildThreadAndGetInParentThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                assertNull(ExecutionContext.getProperties().getProperty("testKey1"));
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }
    
    @Test
    public void testPutInParentThreadPutInChildThreadAndGetInBoth() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                ExecutionContext.getProperties().setProperty("testKey1", "testParentValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        ExecutionContext.getProperties().setProperty("testKey1", "testChildValue1");
                        assertEquals("testChildValue1", ExecutionContext.getProperties().getProperty("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                assertEquals("testParentValue1", ExecutionContext.getProperties().getProperty("testKey1"));
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

    @Test
    public void testGettingAllPropertiesFromThreadHierarchyLeaf() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                ExecutionContext.getProperties().setProperty("testKey1", "testValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        ExecutionContext.getProperties().setProperty("testKey2", "testValue2");
                        ExecutionContext.getProperties().setProperty("testKey3", "testValue3");
                        TestThread tt3 = new TestThread(new Runnable() {
                            @Override
                            public void run() {
                                ExecutionContext.getProperties().setProperty("testKey3", "testValue3Overriden");
                                ExecutionContext.getProperties().setProperty("testKey4", "testValue4");

                                Properties expectedFlattenPropertes = new Properties();
                                expectedFlattenPropertes.put("testKey1", "testValue1");
                                expectedFlattenPropertes.put("testKey2", "testValue2");
                                expectedFlattenPropertes.put("testKey3", "testValue3Overriden");
                                expectedFlattenPropertes.put("testKey4", "testValue4");

                                assertEquals(expectedFlattenPropertes, PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));
                            }
                        });
                        tt3.start();
                        try {
                            tt3.joinExplosively();
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }  
                        
                        Properties expectedFlattenPropertes = new Properties();
                        expectedFlattenPropertes.put("testKey1", "testValue1");
                        expectedFlattenPropertes.put("testKey2", "testValue2");
                        expectedFlattenPropertes.put("testKey3", "testValue3");

                        assertEquals(expectedFlattenPropertes, PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));                        
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                Properties expectedFlattenPropertes = new Properties();
                expectedFlattenPropertes.put("testKey1", "testValue1");

                assertEquals(expectedFlattenPropertes, PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));                        
                
            }
            
        });
        tt1.start();
        tt1.joinExplosively();
        
        Properties expectedFlattenPropertes = new Properties();
        assertEquals(expectedFlattenPropertes, PropertiesHelper.flattenProperties(ExecutionContext.getProperties()));                        
        
    }
    
        
    
    
    
}
