/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.thread;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import org.junit.Before;
import org.junit.Test;

import com.ebmwebsourcing.easycommons.thread.InheritableThreadContext;

/**
 * @author Olivier Fabre - EBM WebSourcing
 */
public class InheritableThreadContextTest {

    @Before
    public void before() {
        InheritableThreadContext.getParameters().clear();
    }

    @Test
    public void testIsEmptyOnInit() throws Exception {
        assertTrue(InheritableThreadContext.getParameters().isEmpty());
    }

    @Test
    public void testIsEmptyAfterPut() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        assertFalse(InheritableThreadContext.getParameters().isEmpty());
    }

    @Test
    public void testGetWithUnknownKey() throws Exception {
        assertNull(InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
    }

    @Test
    public void testPutAndGet() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        assertEquals("testValue1", InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
    }

    @Test
    public void testPutTwiceAndGet() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        assertEquals("testValue1", InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
    }

    @Test
    public void testUpdateAndGet() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        assertEquals("testValue1", InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
        InheritableThreadContext.getParameters().put("testKey1", "testValue2");
        assertEquals("testValue2", InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
    }

    @Test
    public void testClear() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        InheritableThreadContext.getParameters().put("testKey1", "testValue2");
        InheritableThreadContext.getParameters().clear();
        assertTrue(InheritableThreadContext.getParameters().isEmpty());
    }

    @Test
    public void testReset() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        InheritableThreadContext.getParameters().put("testKey1", "testValue2");
        InheritableThreadContext.reset();
        assertTrue(InheritableThreadContext.getParameters().isEmpty());
    }

    @Test
    public void testPutAndGetAfterReset() throws Exception {
        InheritableThreadContext.getParameters().put("testKey1", "oldTestValue1");
        InheritableThreadContext.reset();
        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
        assertEquals("testValue1", InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
    }
    
    @Test
    public void testPutInParentThreadClearInChildThreadAndGetInChildThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                InheritableThreadContext.getParameters().put("testKey1", "testParentValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        InheritableThreadContext.getParameters().put("testKey1", "testChildValue1");
                        assertEquals("testChildValue1", InheritableThreadContext.getParameters()
                                .getValueOrDefault("testKey1"));
                        InheritableThreadContext.getParameters().clear();
                        assertEquals("testParentValue1", InheritableThreadContext.getParameters()
                                .getValueOrDefault("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

    @Test
    public void testPutAndGetInSiblingThreads() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                InheritableThreadContext.getParameters().put("testKey1", "testValue1");
            }
        });
        tt1.start();
        tt1.joinExplosively();

        TestThread tt2 = new TestThread(new Runnable() {
            @Override
            public void run() {
                assertNull(InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
            }
        });
        tt2.start();
        tt2.joinExplosively();
    }

    @Test
    public void testPutInParentThreadAndGetInChildThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                InheritableThreadContext.getParameters().put("testKey1", "testValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        assertEquals("testValue1",
                                InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

    @Test
    public void testPutInChildThreadAndGetInParentThread() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        InheritableThreadContext.getParameters().put("testKey1", "testValue1");
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                assertNull(InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

    @Test
    public void testPutInParentThreadPutInChildThreadAndGetInBoth() throws Exception {
        TestThread tt1 = new TestThread(new Runnable() {
            @Override
            public void run() {
                InheritableThreadContext.getParameters().put("testKey1", "testParentValue1");
                TestThread tt2 = new TestThread(new Runnable() {
                    @Override
                    public void run() {
                        InheritableThreadContext.getParameters().put("testKey1", "testChildValue1");
                        assertEquals("testChildValue1", InheritableThreadContext.getParameters()
                                .getValueOrDefault("testKey1"));
                    }
                });
                tt2.start();
                try {
                    tt2.joinExplosively();
                } catch (InterruptedException e) {
                    throw new RuntimeException(e);
                }
                assertEquals("testParentValue1",
                        InheritableThreadContext.getParameters().getValueOrDefault("testKey1"));
            }
        });
        tt1.start();
        tt1.joinExplosively();
    }

}
