/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the New BSD License (3-clause license).
 *
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the New BSD License (3-clause license)
 * for more details.
 *
 * You should have received a copy of the New BSD License (3-clause license)
 * along with this program/library; If not, see http://directory.fsf.org/wiki/License:BSD_3Clause/
 * for the New BSD License (3-clause license).
 */
package com.ebmwebsourcing.easycommons.thread;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.Semaphore;

/**
 * A task executor that could runs simultaneously all registered tasks.
 * @author Olivier Fabre - EBM WebSourcing
 */
public class SimultaneousTaskExecutor {

    private List<WaitingTask> tasksToExecute = new ArrayList<WaitingTask>();

    /**
     * Register a new task to execute.
     * 
     * @param task
     *            a {@link Runnable} task.
     */
    public void registerTask(Runnable task) {
        this.tasksToExecute.add(new WaitingTask(task));
    }

    /**
     * Clear task list
     */
    public void clearRegisteredTasks() {
        this.tasksToExecute.clear();
    }

    /**
     * Execute all registered tasks simultaneously. Returns when all registered
     * tasks are done.
     * 
     * @throws InterruptedException
     */
    public void executeAllRegisteredTasks() throws InterruptedException {
        final Collection<TestThread> clientPool = new ArrayList<TestThread>();
        final Object starterMonitor = new Object();
        final Semaphore readyMonitor = new Semaphore(tasksToExecute.size());
        for (int i = 0; i < tasksToExecute.size(); i++) {
            readyMonitor.acquire();
            WaitingTask taskToExecute = tasksToExecute.get(i);
            taskToExecute.init(readyMonitor, starterMonitor);
            final TestThread thread = new TestThread(taskToExecute);
            clientPool.add(thread);
            thread.start();
        }

        // Wait that client threads begin to start
        readyMonitor.acquire(tasksToExecute.size());

        // Start all tasks simultaneously
        synchronized (starterMonitor) {
            starterMonitor.notifyAll();
        }

        // Wait the end of processing
        for (TestThread thread : clientPool) {
            thread.joinExplosively();
        }
    }
}
